"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var magic_string_1 = require("magic-string");
var ts = require("typescript");
function rewritePaths(input, rewrite) {
    var output = new magic_string_1.default(input);
    extractPaths(input).forEach(function (path) {
        var result = rewrite(path.id);
        if (result !== path.id && typeof result == 'string') {
            output.overwrite(path.start, path.end, result);
        }
    });
    return output.toString();
}
exports.rewritePaths = rewritePaths;
function extractPaths(input) {
    var paths = [];
    var addPath = function (origin) {
        // Omit the quote characters.
        var start = origin.getStart(module) + 1;
        var end = origin.getEnd() - 1;
        paths.push({
            id: input.slice(start, end),
            start: start,
            end: end,
        });
    };
    var module = ts.createSourceFile('', input, ts.ScriptTarget.Latest);
    module.forEachChild(function (node) {
        if (ts.isImportDeclaration(node)) {
            addPath(node.moduleSpecifier);
        }
        else if (ts.isExportDeclaration(node)) {
            var origin = node.moduleSpecifier;
            if (origin)
                addPath(origin);
        }
        else if (ts.isVariableStatement(node)) {
            for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                var decl = _a[_i];
                var init = decl.initializer;
                if (init && ts.isCallExpression(init)) {
                    var callee = init.expression;
                    if (ts.isIdentifier(callee) && callee.text == 'require') {
                        var origin = init.arguments[0];
                        if (origin && ts.isStringLiteral(origin)) {
                            addPath(origin);
                        }
                    }
                }
            }
        }
    });
    return paths;
}
exports.extractPaths = extractPaths;
